/*  This file is part of the KDE project
    Copyright (C) 2006 Will Stephenson <wstephenson@kde.org>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License version 2 as published by the Free Software Foundation.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

*/

#include "fakenetwork.h"


QList<KNetwork::KIpAddress> FakeNetwork::stringlistToKIpAddress( const QStringList & addrStringList ) const
{
    QList<KNetwork::KIpAddress> ipv4Addrs;
    foreach ( const QString &addrString, addrStringList )
    {
        KNetwork::KIpAddress addr( addrString );
        ipv4Addrs.append( addr );
    }
    return ipv4Addrs;

}

FakeNetwork::FakeNetwork( const QString & uni, const QMap<QString, QVariant> & propertyMap,
                          QObject * parent )
: QObject( parent ), mPropertyMap( propertyMap )
{
    mPropertyMap.insert( "uni", QVariant( uni ) );
}

FakeNetwork::~FakeNetwork()
{

}

QList<KNetwork::KIpAddress> FakeNetwork::ipV4Addresses() const
{
    return stringlistToKIpAddress( mPropertyMap[ "ipv4addresses" ].toStringList() );
}

QList<KNetwork::KIpAddress> FakeNetwork::ipV6Addresses() const
{
    return stringlistToKIpAddress( mPropertyMap[ "ipv6addresses" ].toString().simplified().split( ',' ) );
}

QString FakeNetwork::subnetMask() const
{
    return mPropertyMap[ "subnet" ].toString();
}

QString FakeNetwork::broadcastAddress() const
{
    return mPropertyMap[ "broadcast" ].toString();
}

QString FakeNetwork::route() const
{
    return mPropertyMap[ "route" ].toString();
}

QList<KNetwork::KIpAddress> FakeNetwork::dnsServers() const
{
        return stringlistToKIpAddress( mPropertyMap[ "dns" ].toString().simplified().split( ',' ) );
}

void FakeNetwork::setActivated( bool active )
{
    mPropertyMap.insert( "active", QVariant( active ) );
}

bool FakeNetwork::isActive() const
{
    return mPropertyMap[ "active" ].toBool();
}

QString FakeNetwork::uni() const
{
    return mPropertyMap[ "uni" ].toString();
}

#include "fakenetwork.moc"
